"use strict";
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_builder_1 = require("../node-builder");
const esotope_hammerhead_1 = require("esotope-hammerhead");
const replace_node_1 = __importDefault(require("./replace-node"));
const INVOCATION_FUNC_NAME_RE = /^(call|apply)$/;
// Transform:
// eval.call(ctx, script);
// eval.apply(ctx, script); -->
// eval.call(ctx, __proc$Script(script));
// eval.apply(ctx, __proc$Script(script, true));
const transformer = {
    name: 'eval-call-apply',
    nodeReplacementRequireTransform: false,
    nodeTypes: esotope_hammerhead_1.Syntax.CallExpression,
    condition: node => {
        // eval.<meth>(ctx, script, ...)
        if (node.arguments.length < 2)
            return false;
        if (node.callee.type === esotope_hammerhead_1.Syntax.MemberExpression && node.callee.property.type === esotope_hammerhead_1.Syntax.Identifier &&
            INVOCATION_FUNC_NAME_RE.test(node.callee.property.name)) {
            const obj = node.callee.object;
            // eval.<meth>()
            if (obj.type === esotope_hammerhead_1.Syntax.Identifier && obj.name === 'eval')
                return true;
            // obj.eval.<meth>(), obj[eval].<meth>()
            if (obj.type === esotope_hammerhead_1.Syntax.MemberExpression &&
                (obj.property.type === esotope_hammerhead_1.Syntax.Identifier && obj.property.name ||
                    obj.property.type === esotope_hammerhead_1.Syntax.Literal && obj.property.value) === 'eval')
                return true;
        }
        return false;
    },
    run: node => {
        const callee = node.callee;
        const property = callee.property;
        const newArg = (0, node_builder_1.createProcessScriptMethodCall)(node.arguments[1], property.name === 'apply');
        (0, replace_node_1.default)(node.arguments[1], newArg, node, 'arguments');
        return null;
    },
};
exports.default = transformer;module.exports = exports.default;

