"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const request_event_1 = __importDefault(require("../../session/events/request-event"));
const names_1 = __importDefault(require("../request-hooks/events/names"));
const get_response_1 = __importDefault(require("../request-hooks/response-mock/get-response"));
class BaseRequestPipelineContext {
    constructor(requestId) {
        this.requestFilterRules = [];
        this.requestId = requestId;
    }
    async forEachRequestFilterRule(fn) {
        await Promise.all(this.requestFilterRules.map(fn));
    }
    setupMockIfNecessary(event, eventProvider) {
        const mock = eventProvider.getMock(event.id);
        if (mock && !this.mock)
            this.mock = mock;
    }
    setRequestOptions(eventFactory) {
        this.reqOpts = eventFactory.createRequestOptions();
    }
    async onRequestHookRequest(eventProvider, eventFactory) {
        const requestInfo = eventFactory.createRequestInfo();
        this.requestFilterRules = await eventProvider.getRequestFilterRules(requestInfo);
        await this.forEachRequestFilterRule(async (rule) => {
            const requestEvent = new request_event_1.default({
                requestFilterRule: rule,
                _requestInfo: requestInfo,
                reqOpts: this.reqOpts,
                setMockFn: eventProvider.setMock.bind(eventProvider),
            });
            await eventProvider.callRequestEventCallback(names_1.default.onRequest, rule, requestEvent);
            this.setupMockIfNecessary(requestEvent, eventProvider);
        });
    }
    async getMockResponse() {
        this.mock.setRequestOptions(this.reqOpts);
        return (0, get_response_1.default)(this.mock);
    }
    async handleMockError(eventProvider) {
        const targetRule = this.requestFilterRules[0];
        await eventProvider.callRequestHookErrorHandler(targetRule, this.mock.error);
    }
}
exports.default = BaseRequestPipelineContext;module.exports = exports.default;

